"""init

Revision ID: 0a075bcbeb8f
Revises: 1ab5b6358dd5
Create Date: 2019-06-28 17:50:57.305255

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '0a075bcbeb8f'
down_revision = '1ab5b6358dd5'
branch_labels = None
depends_on = None

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('client', sa.Column('default_scope', sa.Text(), nullable=True))
    # ### end Alembic commands ###

def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('client', 'default_scope')
    # ### end Alembic commands ###
